@@ -186,12 +186,14 @@ class SQLDiff(object):
                     # backwards compatibility for before introspection refactoring (r8296)
                     reverse_type = self.introspection.DATA_TYPES_REVERSE.get(type_code)
             except KeyError:
-                # type_code not found in data_types_reverse map
-                key = (self.differences[-1][:2], description[:2])
-                if key not in self.unknown_db_fields:
-                    self.unknown_db_fields[key] = 1
-                    self.add_difference('comment', "Unknown database type for field '%s' (%s)" % (description[0], type_code))
-                return None
+                reverse_type = self.get_field_db_type_lookup(type_code)
+                if not reverse_type:
+                    # type_code not found in data_types_reverse map
+                    key = (self.differences[-1][:2], description[:2])
+                    if key not in self.unknown_db_fields:
+                        self.unknown_db_fields[key] = 1
+                        self.add_difference('comment', "Unknown database type for field '%s' (%s)" % (description[0], type_code))
+                    return None
 
         kwargs = {}
         if isinstance(reverse_type, tuple):
@@ -221,6 +223,9 @@ class SQLDiff(object):
             field_db_type = getattr(models, reverse_type)(**kwargs).db_type(connection=connection)
         return field_db_type
 
+    def get_field_db_type_lookup(self, type_code):
+        return None
+
     def strip_parameters(self, field_type):
         if field_type and field_type != 'double precision':
             return field_type.split(" ")[0].split("(")[0]
@@ -275,8 +280,7 @@ class SQLDiff(object):
         db_fields = [row[0] for row in table_description]
         for field_name, field in fieldmap.iteritems():
             if field_name not in db_fields:
-                self.add_difference('field-missing-in-db', table_name, field_name, 
-                                                           field.db_type(connection=connection))
+                self.add_difference('field-missing-in-db', table_name, field_name, field.db_type(connection=connection))
 
     def find_field_type_differ(self, meta, table_description, table_name, func=None):
         db_fields = dict([(row[0], row) for row in table_description])
@@ -342,7 +346,7 @@ class SQLDiff(object):
 
             try:
                 table_description = self.introspection.get_table_description(self.cursor, table_name)
-            except Exception, e:
+            except Exception as e:
                 self.add_difference('error', 'unable to introspect table: %s' % str(e).strip())
                 transaction.rollback()  # reset transaction
                 continue
@@ -383,18 +387,18 @@ class SQLDiff(object):
             if not diffs:
                 continue
             if not self.dense and cur_app_label != app_label:
-                print style.NOTICE("+ Application:"), style.SQL_TABLE(app_label)
+                print("%s %s" % (style.NOTICE("+ Application:"), style.SQL_TABLE(app_label)))
                 cur_app_label = app_label
             if not self.dense:
-                print style.NOTICE("|-+ Differences for model:"), style.SQL_TABLE(model_name)
+                print("%s %s" % (style.NOTICE("|-+ Differences for model:"), style.SQL_TABLE(model_name)))
             for diff in diffs:
                 diff_type, diff_args = diff
                 text = self.DIFF_TEXTS[diff_type] % dict((str(i), style.SQL_TABLE(e)) for i, e in enumerate(diff_args))
                 text = "'".join(i % 2 == 0 and style.ERROR(e) or e for i, e in enumerate(text.split("'")))
                 if not self.dense:
-                    print style.NOTICE("|--+"), text
+                    print("%s %s" % (style.NOTICE("|--+"), text))
                 else:
-                    print style.NOTICE("App"), style.SQL_TABLE(app_label), style.NOTICE('Model'), style.SQL_TABLE(model_name), text
+                    print("%s %s %s %s %s" % (style.NOTICE("App"), style.SQL_TABLE(app_label), style.NOTICE('Model'), style.SQL_TABLE(model_name), text))
 
     def print_diff_sql(self, style):
         cur_app_label = None
@@ -402,24 +406,24 @@ class SQLDiff(object):
         has_differences = max([len(diffs) for app_label, model_name, diffs in self.differences])
         if not has_differences:
             if not self.dense:
-                print style.SQL_KEYWORD("-- No differences")
+                print(style.SQL_KEYWORD("-- No differences"))
         else:
-            print style.SQL_KEYWORD("BEGIN;")
+            print(style.SQL_KEYWORD("BEGIN;"))
             for app_label, model_name, diffs in self.differences:
                 if not diffs:
                     continue
                 if not self.dense and cur_app_label != app_label:
-                    print style.NOTICE("-- Application: %s" % style.SQL_TABLE(app_label))
+                    print(style.NOTICE("-- Application: %s" % style.SQL_TABLE(app_label)))
                     cur_app_label = app_label
                 if not self.dense:
-                    print style.NOTICE("-- Model: %s" % style.SQL_TABLE(model_name))
+                    print(style.NOTICE("-- Model: %s" % style.SQL_TABLE(model_name)))
                 for diff in diffs:
                     diff_type, diff_args = diff
                     text = self.DIFF_SQL[diff_type](style, qn, diff_args)
                     if self.dense:
                         text = text.replace("\n\t", " ")
-                    print text
-            print style.SQL_KEYWORD("COMMIT;")
+                    print(text)
+            print(style.SQL_KEYWORD("COMMIT;"))
 
 
 class GenericSQLDiff(SQLDiff):
@@ -501,6 +505,10 @@ class PostgresqlSQLDiff(SQLDiff):
         55902: 'django.contrib.gis.db.models.fields.MultiPolygonField',
     }
 
+    DATA_TYPES_REVERSE_NAME = {
+        'hstore': 'django_hstore.hstore.DictionaryField',
+    }
+
     # Hopefully in the future we can add constraint checking and other more
     # advanced checks based on this database.
     SQL_LOAD_CONSTRAINTS = """
@@ -564,6 +572,14 @@ class PostgresqlSQLDiff(SQLDiff):
                     self.add_difference('notnull-differ', table_name, field.name, action)
         return db_type
 
+    @transaction.autocommit
+    def get_field_db_type_lookup(self, type_code):
+        try:
+            name = self.sql_to_dict("SELECT typname FROM pg_type WHERE typelem=%s;", [type_code])[0]['typname']
+            return self.DATA_TYPES_REVERSE_NAME.get(name.strip('_'))
+        except (IndexError, KeyError):
+            pass
+
     """
     def find_field_type_differ(self, meta, table_description, table_name):
         def callback(field, description, model_type, db_type):
@@ -575,7 +591,7 @@ class PostgresqlSQLDiff(SQLDiff):
 
 DATABASE_SQLDIFF_CLASSES = {
     'postgis': PostgresqlSQLDiff,
-    'postgresql_psycopg2' : PostgresqlSQLDiff,
+    'postgresql_psycopg2': PostgresqlSQLDiff,
     'postgresql': PostgresqlSQLDiff,
     'mysql': MySQLDiff,
     'sqlite3': SqliteSQLDiff,
@@ -622,9 +638,9 @@ to check/debug ur models compared to the real database tables and columns."""
         if engine == 'dummy':
             # This must be the "dummy" database backend, which means the user
             # hasn't set DATABASE_ENGINE.
-            raise CommandError("Django doesn't know which syntax to use for your SQL statements,\n" +
-                "because you haven't specified the DATABASE_ENGINE setting.\n" +
-                "Edit your settings file and change DATABASE_ENGINE to something like 'postgresql' or 'mysql'.")
+            raise CommandError("""Django doesn't know which syntax to use for your SQL statements,
+because you haven't specified the DATABASE_ENGINE setting.
+Edit your settings file and change DATABASE_ENGINE to something like 'postgresql' or 'mysql'.""")
 
         if options.get('all_applications', False):
             app_models = models.get_models()
@@ -633,7 +649,7 @@ to check/debug ur models compared to the real database tables and columns."""
                 raise CommandError('Enter at least one appname.')
             try:
                 app_list = [models.get_app(app_label) for app_label in app_labels]
-            except (models.ImproperlyConfigured, ImportError), e:
+            except (models.ImproperlyConfigured, ImportError) as e:
                 raise CommandError("%s. Are you sure your INSTALLED_APPS setting is correct?" % e)
 
             app_models = []
